﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using ParserLib;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;

namespace LogParser
{
    internal class Program
    {
        static void Display(Dictionary<string, int> amounts, Dictionary<string, int> daysAmounts)
        {
            Dictionary<string, long> score = new Dictionary<string, long>();
            long sum = 0;


            foreach (KeyValuePair<string, int> pair in amounts)
            {
                score[pair.Key] = amounts[pair.Key] * daysAmounts[pair.Key];
            }

            IOrderedEnumerable<KeyValuePair<string, long>> ordered = score.OrderBy(x => x.Value);

            foreach (KeyValuePair<string, long> pair in ordered)
            {
                Console.WriteLine(pair.Key + ", Score: " + pair.Value + ((daysAmounts[pair.Key] == 1) ? ", Day: " : ", Days: ") + daysAmounts[pair.Key] + ", Amounts: " + amounts[pair.Key]);
                sum += amounts[pair.Key];
            }

            Console.WriteLine();
            Console.WriteLine("All: " + (sum - amounts[ordered.Last().Key]));
            Console.WriteLine("Unique: " + (ordered.ToList().Count - 1));
        }

        static void Main(string[] args)
        {
            if (args.Length != 1 && args.Length != 2)
            {
                Console.WriteLine("LogParser.exe -text \"text\"");
                Console.WriteLine("LogParser.exe -ip");
                Console.WriteLine("LogParser.exe -both \"text\"");
                Console.WriteLine("LogParser.exe -urls");
                Console.ReadLine();
                return;
            }
            else if (args[0] == "-text")
            {
                List<string> lines;

                lines = Parser.GetLines("./");
                Console.WriteLine("Occurrences Count of \"" + args[1] + "\" : " + Parser.CountOccurences(lines, args[1]));
                Console.ReadLine();
                return;
            }
            else if (args[0] == "-ip")
            {
                Dictionary<string, int> dictAmounts;
                Dictionary<string, int> dictDaysAmounts;
                List<string> lines;

                lines = Parser.GetLines("./");
                Parser.GetIpAddresses(lines, "", out dictAmounts, out dictDaysAmounts);
                Display(dictAmounts, dictDaysAmounts);

                Console.ReadLine();
                return;
            }
            else if (args[0] == "-both")
            {
                Dictionary<string, int> dictAmounts;
                Dictionary<string, int> dictDaysAmounts;
                List<string> lines;

                lines = Parser.GetLines("./");
                Parser.GetIpAddresses(lines, args[1], out dictAmounts, out dictDaysAmounts);
                Display(dictAmounts, dictDaysAmounts);

                Console.ReadLine();
                return;
              }
            else if (args[0] == "-urls")
            {
                Dictionary<string, int> dict;
                long sum = 0;
                List<string> lines;

                lines = Parser.GetLines("./");
                Parser.GetUrls(lines, out dict);
                IOrderedEnumerable<KeyValuePair<string, int>> ordered = dict.OrderBy(x => x.Value);

                foreach (KeyValuePair<string, int> pair in ordered)
                {
                    Console.WriteLine(pair.Key + " : " + pair.Value);
                    sum += pair.Value;
                }

                Console.WriteLine();
                Console.WriteLine("All : " + (sum - ordered.Last().Value));
                Console.WriteLine("Unique : " + (ordered.ToList().Count));
                Console.ReadLine();
                return;
            }
        }
    }
}
