﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;

namespace ParserLib
{
    internal class Ip
    {
        public string Address { get; set; }
        public DateTime Time { get; set; }
    }

    public class Parser
    {
        static List<FileInfo> GetFileNames(string dir)
        {
            List<FileInfo> list;
            DirectoryInfo directoryInfo = new DirectoryInfo(dir);

            list = new List<FileInfo>(directoryInfo.EnumerateFiles());
            return list;
        }

        static int CountOccurences(FileInfo fileInfo, string str)
        {
            int count = 0;
            StreamReader streamReader = new StreamReader(fileInfo.FullName);
            string line;

            while ((line = streamReader.ReadLine()) != null)
            {
                if (line.Contains(str))
                {
                    count++;
                }
            }
            return count;
        }

        static List<Ip> GetIpAddresses(string line, string word)
        {
            List<Ip> list = new List<Ip>();
         
            if (word == "" || (line.Contains(word) && word != ""))
            {
                list.AddRange(GetIpAddresses(line));
            }
            return list;
        }

        static List<string> GetUrls(List<string> lines)
        {
            List<string> list = new List<string>();

            foreach (string line in lines)
            {
                if (line.Contains("Home"))
                {
                    list.AddRange(GetUrls(line));
                }
            }
            return list;
        }

        static List<string> GetLines(FileInfo fileInfo)
        {
            List<string> list = new List<string>();
            StreamReader streamReader = new StreamReader(fileInfo.FullName);
            string line;

            while ((line = streamReader.ReadLine()) != null)
            {
                list.Add(line);
            }
            return list;
        }

        static DateTime GetTime(string line)
        {
            DateTime time = DateTime.MinValue;

            try
            {
                time = DateTime.Parse(line);
            }
            catch (Exception)
            {

            }
            return time;
        }

        static List<Ip> GetIpAddresses(string line)
        {
            List<Ip> list = new List<Ip>();
            Match match = Regex.Match(line, @"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}");
            string[] arr = line.Split(' ');

            while (match.Success)
            {
                Ip ip = new Ip();

                ip.Time = GetTime(arr[0] + ' ' + arr[1]);
                ip.Address = match.Value;
                list.Add(ip);
                match = match.NextMatch();
            }
            return list;
        }

        static List<string> GetUrls(string line)
        {
            List<string> list = new List<string>();
            string[] arr = line.Split(' ');
            List<Ip> ip = GetIpAddresses(line);

            if (ip.Count > 1 && ip[1].Address != "127.0.0.1" && ip[1].Address != "85.190.241.58" && ip[1].Address != "75.119.132.197" && ip[1].Address != "62.146.178.120" && arr.Length > 5 && arr[3] == "GET" && !line.Contains("1::") && !line.Contains(".jpg"))
            {
                list.Add(arr[4]);
            }
            return list;
        }

        static int AmountOfDifferentDays(List<DateTime> times)
        {
            int amount = 1;
            DateTime now;

            times.Sort();
            now = times[0];

            foreach (DateTime dt in times)
            {
                if ((dt - now).TotalHours > 24)
                {
                    now = dt;
                    amount++;
                }
            }
            return amount;
        }

        public static void GetIpAddresses(List<string> lines, string word, out Dictionary<string, int> dictAmounts, out Dictionary<string, int> dictDaysAmounts)
        {
            List<Ip> addr;
            Dictionary<string, List<DateTime>> dictTimes = new Dictionary<string, List<DateTime>>();
            dictAmounts = new Dictionary<string, int>();
            dictDaysAmounts = new Dictionary<string, int>();

            foreach (string line in lines)
            {
                addr = GetIpAddresses(line, word);
                foreach (Ip str in addr)
                {
                    if (!dictAmounts.Keys.Contains(str.Address))
                    {
                        dictAmounts.Add(str.Address, 0);
                        dictTimes.Add(str.Address, new List<DateTime>());
                    }
                    dictAmounts[str.Address]++;
                    dictTimes[str.Address].Add(str.Time);
                }
            }

            IOrderedEnumerable<KeyValuePair<string, int>> ordered = dictAmounts.OrderBy(x => x.Value);

            foreach (KeyValuePair<string, int> pair in ordered)
            {
                dictDaysAmounts[pair.Key] = AmountOfDifferentDays(dictTimes[pair.Key]);
            }
        }

        public static void GetUrls(List<string> lines, out Dictionary<string, int> dict)
        {
            List<string> urls;
            dict = new Dictionary<string, int>();

            urls = GetUrls(lines);
            foreach (string str in urls)
            {
                if (!dict.Keys.Contains(str))
                {
                    dict.Add(str, 0);
                }
                dict[str]++;
            }
        }

        public static List<string> GetLines(string dir)
        {
            List<string> lines = new List<string>();
            List<FileInfo> list = GetFileNames(dir);

            foreach (FileInfo fileInfo in list)
            {
                lines.AddRange(GetLines(fileInfo));
            }
            return lines;
        }

        public static int CountOccurences(List<string> lines, string word)
        {
            int occ = 0;

            foreach(string line in lines)
            {
                if(line.Contains(word))
                {
                    occ++;
                }
            }
            return occ;
        }
    }
}
