﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using CounterLib;
using ParserLib;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.ServiceProcess;
using System.Timers;
using Timer = System.Timers.Timer;


namespace WindowsServiceLogs
{
    public partial class Service : ServiceBase
    {
        protected Timer m_Timer;
        protected string LogDirectory = "C:\\inetpub_logs\\logs\\";
        protected string ResultsDirectory = "C:\\inetpub_logs\\results\\";
        protected string Ip = "ip.text";
        protected string Urls = "urls.text";
        protected string Words = "words.text";
        protected string LogFile = "C:\\inetpub_logs\\exe\\WindowsServiceLogs.log";
        protected StreamWriter writerLogger;

        public Service()
        {
            InitializeComponent();
        }

        protected void InitLogger()
        {
            writerLogger = new StreamWriter(LogFile);
        }

        protected void Log(string msg)
        {
            writerLogger.WriteLine(DateTime.Now.ToString() + ", " + msg);
        }

        protected void FlushLogger()
        {
            writerLogger.Flush();
        }

        protected void DisposeLogger()
        {
            writerLogger.Dispose();
        }

        protected List<string> GetWords(string file)
        {
            try
            {
                Log("Begin GetWords: " + file);
                List<string> words = new List<string>();
                StreamReader reader = new StreamReader(file);
                string word;

                while ((word = reader.ReadLine()) != null)
                {
                    words.Add(word);
                }
                Log("End GetWords: " + file);
                return words;
            }
            catch(Exception ex)
            {
                Log("Exception GetWords: " + ex.Message);
                return null;
            }
        }

        protected void Write(Dictionary<string, int> amounts, Dictionary<string, int> daysAmounts, string file)
        {
            try
            {
                Log("Begin Write: " + file);
                if (File.Exists(file))
                {
                    File.Delete(file);
                }

                StreamWriter writer = new StreamWriter(file);

                Dictionary<string, long> score = new Dictionary<string, long>();
                long sum = 0;

                foreach (KeyValuePair<string, int> pair in amounts)
                {
                    score[pair.Key] = amounts[pair.Key] * daysAmounts[pair.Key];
                }

                IOrderedEnumerable<KeyValuePair<string, long>> ordered = score.OrderBy(x => x.Value);

                foreach (KeyValuePair<string, long> pair in ordered)
                {
                    writer.WriteLine(pair.Key + ", Score: " + pair.Value + ((daysAmounts[pair.Key] == 1) ? ", Day: " : ", Days: ") + daysAmounts[pair.Key] + ", Amounts: " + amounts[pair.Key]);
                    sum += amounts[pair.Key];
                }

                writer.WriteLine();
                writer.WriteLine("All: " + (sum - amounts[ordered.Last().Key]));
                writer.WriteLine("Unique: " + (ordered.ToList().Count - 1));

                writer.Flush();
                writer.Close();
                Log("End Write: " + file);
            }
            catch (Exception ex)
            {
                Log("Exception Write: " + ex.Message);
            }
        }

        protected void Write(Dictionary<string, int> amounts, string file)
        {
            try
            {
                Log("Begin Write: " + file);
                if (File.Exists(file))
                {
                    File.Delete(file);
                }

                StreamWriter writer = new StreamWriter(file);
                long sum = 0;

                IOrderedEnumerable<KeyValuePair<string, int>> ordered = amounts.OrderBy(x => x.Value);

                foreach (KeyValuePair<string, int> pair in ordered)
                {
                    writer.WriteLine(pair.Key + ", Amounts: " + amounts[pair.Key]);
                    sum += amounts[pair.Key];
                }

                writer.WriteLine();
                writer.WriteLine("All: " + (sum - amounts[ordered.Last().Key]));
                writer.WriteLine("Unique: " + ordered.ToList().Count);

                writer.Flush();
                writer.Close();
                Log("End Write: " + file);
            }
            catch(Exception ex)
            {
                Log("Exception Write: " + ex.Message);
            }
        }

        protected void Parse(List<string> lines, string word, string file)
        {
            try
            {
                Log("Begin Parse: " + file + ", word: " + word);
                Dictionary<string, int> dictAmounts;
                Dictionary<string, int> dictDaysAmounts;

                Parser.GetIpAddresses(lines, word, out dictAmounts, out dictDaysAmounts);
                Write(dictAmounts, dictDaysAmounts, ResultsDirectory + file);
                Log("End Parse: " + file + ", word: " + word);
            }
            catch(Exception ex)
            {
                Log("Exception Parse: " + ex.Message);
            }
        }

        protected void ParseUrls(List<string> lines)
        {
            try
            {
                Log("Begin ParseUrls: " + lines.Count);
                Dictionary<string, int> dictAmounts;

                Parser.GetUrls(lines, out dictAmounts);
                Write(dictAmounts, ResultsDirectory + Urls);
                Log("End ParseUls: " + lines.Count);
            }
            catch(Exception ex)
            {
                Log("Exception ParseUrls: " + ex.Message);
            }
        }


        protected void Run(object ob, ElapsedEventArgs eea)
        {
            try
            {
                Log("Begin Run.");
                List<string> lines;
                List<string> words;

                Copier.CopyAllFiles("C:\\inetpub\\logs\\LogFiles\\W3SVC2\\", LogDirectory);
                lines = Parser.GetLines(LogDirectory);

                Parse(lines, "", Ip);
                ParseUrls(lines);

                words = GetWords(ResultsDirectory + Words);
                foreach(string word in words)
                {
                    Parse(lines, word, word + ".text");
                }
                Log("End Run.");
                FlushLogger();
            }
            catch(Exception ex)
            {
                Log("Exception Run: " + ex.Message);
            }
        }

        protected override void OnStart(string[] args)
        {
            InitLogger();
            Run(null, null);
            m_Timer = new Timer
            {
                Interval = 60000 * 60
            };
            m_Timer.Elapsed += new ElapsedEventHandler(Run);
            m_Timer.Start();
        }

        protected override void OnStop()
        {
            try
            {
                Log("Stopping.");
                m_Timer.Dispose();
                FlushLogger();
                DisposeLogger();
            }
            catch(Exception ex)
            {

            }
        }
    }
}
